@@ -1,8 +1,8 @@
 """
 API arguments and calls
 
->>> Setup.from_command_line("--save --update --archives-from-proxy --sources bullseye --sources-from-origin-with-lts Debian --chroots-from-sources --repositories test/test").command_line()
-'--save --update --archives-from-proxy --sources bullseye --sources-from-origin-with-lts Debian --chroots-from-sources --repositories test/test'
+>>> Setup.from_command_line("--save --update all --archives-from-proxy --sources bullseye --sources-from-origin-with-lts Debian --chroots-from-sources --repositories test/test").command_line()
+'--save --update all --archives-from-proxy --sources bullseye --sources-from-origin-with-lts Debian --repositories test/test --chroots-from-sources'
 
 >>> Setup.from_sloppy_args(save=True, archives_from_proxy=True, archives_from_apt_sources=False).command_line()
 '--save --archives-from-proxy'
@@ -18,6 +18,9 @@ import argparse
 import re
 import glob
 import shlex
+import hashlib
+import json
+import threading
 from contextlib import closing
 
 import mini_buildd.misc
@@ -343,6 +346,8 @@ class Call():
     CONFIRM = False
     CUSTOM_HTML_SNIPPET = None
 
+    _LOCK = threading.Lock()  # Optional use
+
     @classmethod
     def name(cls):
         return mini_buildd.misc.Snake(cls.__name__).from_camel()
@@ -384,9 +389,7 @@ class Call():
         keys = [a.identity for a in cls.iarguments()]
         return cls(**{key: value for key, value in kwargs.items() if key in keys})
 
-    def __init__(self, **kwargs):
-        self.args = {arg.identity: arg for arg in self.iarguments()}
-
+    def set_args(self, **kwargs):
         LOG.debug(f"{self.name()}: Given args: {kwargs}")
         for key, value in kwargs.items():
             if key not in self.args:
@@ -397,6 +400,9 @@ class Call():
         for key in (key for key in self.args if key in kwargs):
             self.args[key].set(kwargs.get(key))
 
+    def __init__(self, **kwargs):
+        self.args = {arg.identity: arg for arg in self.iarguments()}
+        self.set_args(**kwargs)
         self.request = None
         self.result = {}  # json result
 
@@ -432,21 +438,23 @@ class Call():
             parser.add_argument(*argument.id_list, **argument.argparse_kvsargs())
         return vars(parser.parse_args(args=shlex.split(command_line)))
 
-    def icommand_line(self, full=False, with_user=False, user=None):
+    def icommand_line(self, full=False, with_user=False, user=None, exclude=None):
+        _exclude = [] if exclude is None else exclude
+
         if full:
             yield "mini-buildd-api"
             yield self.name()
             yield mini_buildd.http_endpoint().geturl(with_user=with_user, user=user)
-        for arg in self.args.values():
-            if arg.given is not None:
-                if arg.is_positional:
-                    yield from arg.icommand_line_given()
-                else:
-                    yield arg.id_list[0]
-                    yield from arg.icommand_line_given()
 
-    def command_line(self, full=False, with_user=False, user=None):
-        return mini_buildd.PyCompat.shlex_join(self.icommand_line(full=full, with_user=with_user, user=user))
+        for arg in (a for a in self.args.values() if a.identity not in _exclude and a.given is not None):
+            if arg.is_positional:
+                yield from arg.icommand_line_given()
+            else:
+                yield arg.id_list[0]
+                yield from arg.icommand_line_given()
+
+    def command_line(self, full=False, with_user=False, user=None, exclude=None):
+        return mini_buildd.PyCompat.shlex_join(self.icommand_line(full=full, with_user=with_user, user=user, exclude=exclude))
 
     def command_line_full(self):
         user = self.request.user if self.AUTH is not mini_buildd.config.AUTH_NONE and self.request is not None and self.request.user.is_authenticated else None
@@ -1011,7 +1019,7 @@ class Handshake(Call):
                 LOG.debug(f"Remote handshake ok: '{r}': {r.key_long_id}: {r.key_name}")
                 return
             except Exception as e:
-                LOG.debug(f"Remote handshake failed for '{r}': {e}")
+                mini_buildd.log_exception(LOG, f"Remote handshake failed for '{r}'", e)
         raise mini_buildd.HTTPBadRequest(f"GnuPG handshake failed: No remote for public key on {mini_buildd.http_endpoint()}")
 
 
@@ -1251,7 +1259,8 @@ class Setup(_Admin, _Confirm, Call):
     @classmethod
     def iarguments(cls):
         yield BoolArgument(["--save"], doc="Save command line from this (successful) run as default for future runs", header="Run Options")
-        yield BoolArgument(["--update"], doc="Update existing instances")
+        yield StrArgument(["--update"], default="", doc="Update existing instances; 'all' to update all, empty string to never update, <hash_id>,.. (see a previous run) to update selected instances")
+        yield StrArgument(["--pca"], default="", doc="Prepare, check and create instances; 'all' to pca all, empty string to never pca, <hash_id>,.. (see a previous run) to pca selected instances")
 
         yield StrArgument(["--identity"], default=Value.DEFAULT_IDENTITY, doc="instance identity (for keyring package names, dput config, ...)", header="Daemon")
         yield StrArgument(["--ftp-endpoint"], default=Value.DEFAULT_FTP_ENDPOINT, doc="ftp (incoming) network endpoint")
@@ -1266,6 +1275,13 @@ class Setup(_Admin, _Confirm, Call):
         yield ListArgument(["--sources-from-origin-with-lts"], default=[], choices=SETUP["origin"].keys(), doc="add 'supported' plus 'lts' (as per ``distro-info`` -- adds 'lts', 'elts' for Debian and 'esm' for Ubuntu) set of source codenames from these origins")
         yield ListArgument(["--sources-from-origin-with-all"], default=[], choices=SETUP["origin"].keys(), doc="add all source codenames with a working setup from these origins (i.e., includes very old ones)")
 
+        yield ListArgument(["--repositories"], default=[], doc=f"repositories to setup \'<repo_id>[/({'|'.join(SETUP['repository'].keys())})]\'", header="Repositories")
+        yield ListArgument(["--repositories-extra-architectures"], default=[],
+                           doc="""add these extra architectures to distributions
+
+We will always add all architectures your host natively supports.
+""")
+
         yield ListArgument(["--chroots"], default=[], choices=CODENAMES.keys(), doc="codenames to setup chroots for", header="Chroots")
         yield BoolArgument(["--chroots-from-sources"], doc="add chroots for all base sources found")
         yield ListArgument(["--chroots-extra-architectures"], default=[], choices=mini_buildd.dist.Archs.available(),
@@ -1282,40 +1298,94 @@ Chroots that cannot be build for a given architecture will just be skipped.
 
         yield ListArgument(["--remotes"], default=[], doc=f"remotes to add. {inspect.getdoc(mini_buildd.net.ClientEndpoint)}", header="Remotes")
 
-        yield ListArgument(["--repositories"], default=[], doc=f"repositories to setup \'<repo_id>[/({'|'.join(SETUP['repository'].keys())})]\'", header="Repositories")
-        yield ListArgument(["--repositories-extra-architectures"], default=[],
-                           doc="""add these extra architectures to distributions
+    class Instance:
+        def diff(self):
+            diff = {
+                "setup": {
+                    "fields": [],
+                    "diff": {},
+                },
+                "model": {
+                    "fields": [],
+                    "diff": {},
+                },
+            }
 
-We will always add all architectures your host natively supports.
-""")
+            setup = {**self.identifiers, **self.options}
+            target = self.model(**setup)
+            for field in (f for f in self.model.mbd_get_fields(exclude=["id", "status", "last_checked", "auto_reactivate", "pickled_data", "ping"]) if not f.is_relation):
+                data = diff["setup"] if field.name in setup else diff["model"]
 
-    def x_or_create(self, cls, defaults=None, **kwargs):
-        _defaults = {} if defaults is None else defaults
+                data["fields"].append(field.name)
+                current_value = None if self.obj is None else getattr(self.obj, field.name, None)
+                target_value = getattr(target, field.name, None)
+                if current_value != target_value:
+                    data["diff"][field.name] = {
+                        "current": str(current_value),
+                        "target": str(target_value),
+                    }
+            return diff
 
-        info = {
-            "updated": False,
-            "created": False,
-            "diff": {},
-            "desc": "",
-        }
+        @classmethod
+        def _add_m2m_diff(cls, diff, field, expected, current):
+            diff["setup"]["fields"].append(field)
+            _expected = set(expected)
+            _current = set(current)
+            if _expected - _current:
+                diff["setup"]["diff"].setdefault(field, {})["missing"] = list(_expected - _current)
+            if _current - _expected:
+                diff["setup"]["diff"].setdefault(field, {})["unknown"] = list(_current - _expected)
 
-        obj = None
-        try:
-            LOG.debug(f"Found {cls.__name__} ({kwargs})")
-            obj = cls.objects.get(**kwargs)
-            info["diff"] = cls.mbd_diff(obj, **kwargs, **_defaults)
-            if self.args["update"].value():
-                info["updated"] = cls.objects.filter(**kwargs).update(**_defaults) > 0
-        except cls.DoesNotExist:
-            LOG.debug(f"NOT found {cls.__name__} ({kwargs})")
-            obj = cls(**kwargs, **_defaults)
-            obj.save()
-            info["created"] = True
+        def __init__(self, call, model, options, update_args, **identifiers):
+            self.call, self.model, self.options, self.update_args, self.identifiers = call, model, options, update_args, identifiers
+            LOG.debug(f"Instance: {self.model}({self.identifiers})")
 
-        info["desc"] = str(obj)
+            self.identity = {
+                "class": repr(self.model),
+                "identifiers": {k: repr(v) for k, v in self.identifiers.items()},
+            }
+            self.identity_hash = hashlib.sha256(json.dumps(self.identity, sort_keys=True).encode(mini_buildd.config.CHAR_ENCODING)).hexdigest()
 
-        self.result.setdefault(cls.__name__, {})[obj.pk] = info
-        return obj, info["created"]
+            self.obj = model.mbd_get_or_none(**self.identifiers)
+            self.info = {
+                "identity": self.identity,
+                "identity_hash": self.identity_hash,
+                "identity_str": f"{self.model.__name__}({self.identifiers})",
+                "updated": False,
+                "created": False,
+                "pca": False,
+                "diff": None,
+                "status": None,
+            }
+
+            if self.call.args["update"].value() in ["all"] + self.identity_hash.split(","):
+                if self.obj is None:
+                    self.obj = self.model(**self.identifiers, **self.options)
+                    self.obj.save()
+                    self.info["created"] = True
+                else:
+                    self.info["updated"] = self.model.objects.filter(**self.identifiers).update(**self.options) > 0
+                    self.obj = self.model.mbd_get_or_none(**self.identifiers)  # Get updated instance
+
+                if self.obj is not None:
+                    self.update()
+                    self.obj.save()
+
+            if self.call.args["pca"].value() in ["all"] + self.identity_hash.split(","):
+                if self.obj is not None and issubclass(self.model, mini_buildd.mdls().base.StatusModel):
+                    self.model.Admin.mbd_pca(self.call.request, [self.obj])
+                    self.info["pca"] = True
+
+            self.info["diff"] = self.diff()
+            self.info["status"] = "None" if self.obj is None else self.obj.get_status_display() if issubclass(self.model, mini_buildd.mdls().base.StatusModel) else "Exists"
+            if self.obj is not None and not self.model.__name__.endswith("Option"):
+                # Help for HTML template only
+                self.info["change_instance_url"] = f"admin:mini_buildd_{self.model.__name__.lower()}_change"
+                self.info["change_instance_pk"] = str(self.obj.pk)
+            self.call.result.append(self.info)
+
+        def update(self):
+            pass
 
     @classmethod
     def ilocal_archive_urls(cls):
@@ -1337,174 +1407,183 @@ We will always add all architectures your host natively supports.
                     LOG.debug(f"Local proxy archive: '{url}/{path}/'")
                     yield f"{url}/{path}/"
 
-    def setup_daemon(self):
-        defaults = {
-            "identity": self.args["identity"].value(),
-            "ftpd_bind": self.args["ftp_endpoint"].value(),
-        }
+    def _check_codenames(self, codenames):
+        for codename in codenames:
+            if CODENAMES.get(codename) is None:
+                raise mini_buildd.HTTPBadRequest(f"No setup for codename '{codename}'")
 
-        self.x_or_create(mini_buildd.mdls().daemon.Daemon, defaults=defaults)
+    def _setup(self):
+        # Daemon
+        Setup.Instance(self,
+                       mini_buildd.mdls().daemon.Daemon,
+                       {
+                           "identity": self.args["identity"].value(),
+                           "ftpd_bind": self.args["ftp_endpoint"].value(),
+                       },
+                       {})
 
-        mini_buildd.mdls().daemon.Daemon.Admin.mbd_pca_all(self.request)
-
-    def setup_archives(self):
-        archive_urls = []
-        archive_urls += self.args["archives"].value()
+        # Archives
+        archive_urls = set(self.args["archives"].value())
         if self.args["archives_from_apt_sources"].value():
-            archive_urls += self.ilocal_archive_urls()
+            archive_urls.update(self.ilocal_archive_urls())
         if self.args["archives_from_proxy"].value():
-            archive_urls += self.iapt_cacher_archive_urls()
+            archive_urls.update(self.iapt_cacher_archive_urls())
         for v in self.args["archives_from_origin"].value():
-            archive_urls += SETUP["origin"][v]["archive"]
+            archive_urls.update(SETUP["origin"][v]["archive"])
 
         for url in archive_urls:
-            self.x_or_create(mini_buildd.mdls().source.Archive, url=url)
+            Setup.Instance(self, mini_buildd.mdls().source.Archive, {}, {}, url=url)
 
-    def setup_sources(self):
-        codenames = []
-        codenames += self.args["sources"].value()
+        # Sources
+        sources_codenames = set(self.args["sources"].value())
         for v in self.args["sources_from_origin"].value():
-            codenames += CODENAMES.get_supported(v)
+            sources_codenames.update(CODENAMES.get_supported(v))
         for v in self.args["sources_from_origin_with_lts"].value():
-            codenames += CODENAMES.get_supported_plus_lts(v)
+            sources_codenames.update(CODENAMES.get_supported_plus_lts(v))
         for v in self.args["sources_from_origin_with_all"].value():
-            codenames += CODENAMES.get_working(v)
+            sources_codenames.update(CODENAMES.get_working(v))
+        self._check_codenames(sources_codenames)
 
-        for codename in set(codenames):
-            for s in CODENAMES[codename]["sources"]:
-                obj, created = self.x_or_create(mini_buildd.mdls().source.Source, origin=s.origin, codename=s.codename, defaults={"extra_options": s.extra_options})
-                if created or self.args["update"].value():
-                    # apt_keys (m2m) need to be updated manually, with save() (x_or_create can't be used)
-                    obj.apt_keys.clear()
-                    for long_key_id in s.apt_keys:
-                        matching_keys = mini_buildd.mdls().gnupg.AptKey.mbd_filter_key(long_key_id)
-                        if matching_keys:
-                            apt_key = matching_keys[0]
-                        else:
-                            apt_key, _created = self.x_or_create(mini_buildd.mdls().gnupg.AptKey, key_id=long_key_id)
-                        obj.apt_keys.add(apt_key)
-                    obj.save()
+        class SourceInstance(Setup.Instance):
+            def update(self):
+                # apt_keys (m2m) need to be updated manually, with save() (x_or_create can't be used)
+                self.obj.apt_keys.clear()
+                for long_key_id in self.update_args["apt_keys"]:
+                    matching_keys = mini_buildd.mdls().gnupg.AptKey.mbd_filter_key(long_key_id)
+                    if matching_keys:
+                        self.obj.apt_keys.add(matching_keys[0])
+                    else:
+                        self.obj.apt_keys.add(Setup.Instance(self.call, mini_buildd.mdls().gnupg.AptKey, {}, {}, key_id=long_key_id).obj)
 
+            def diff(self):
+                diff = super().diff()
+                if self.obj is not None:
+                    self._add_m2m_diff(diff, "apt_keys", self.update_args["apt_keys"], [a.key_long_id for a in self.obj.apt_keys.all()])
+                return diff
+
+        for codename in sources_codenames:
+            for c, s in CODENAMES[codename]["sources"].items():
+                origin = s.get("origin", CODENAMES[codename]["origin"])
+                source = SourceInstance(self, mini_buildd.mdls().source.Source, s.get("options", {}), {"apt_keys": s.get("apt_keys", CODENAMES[codename].get("apt_keys"))}, origin=origin, codename=c)
                 # Smartly add prio source: Security gets prio=500, others prio=1 (opt-in).
-                self.x_or_create(mini_buildd.mdls().source.PrioritySource, source=obj, priority=500 if CODENAMES.is_security(obj.origin, obj.codename) else 1)
+                Setup.Instance(self, mini_buildd.mdls().source.PrioritySource, {}, {}, source=source.obj, priority=500 if CODENAMES.is_security(origin, c) else 1)
 
-        mini_buildd.mdls().source.Source.Admin.mbd_pca_all(self.request)
-
-    def setup_chroots(self):
-        codenames = []
-        codenames += self.args["chroots"].value()
-        if self.args["chroots_from_sources"].value():
-            codenames += [s.codename for s in mini_buildd.mdls().source.Source.Admin.mbd_filter_active_base_sources()]
-
-        # Setup chroots
-        for codename in codenames:
-            source = mini_buildd.mdls().source.Source.Admin.mbd_filter_active_base_sources().get(codename=codename)
-            cb_class = getattr(mini_buildd.mdls().chroot, f"{self.args['chroots_backend'].value()}Chroot")
-
-            for architecture in source.architectures.all():
-                if architecture.name in (mini_buildd.dist.Archs.native() + self.args["chroots_extra_architectures"].value()):
-                    try:
-                        defaults = {}
-                        if CODENAMES[source.codename].get("needs_uname_26", False):
-                            defaults["extra_options"] = "Debootstrap-Command: /usr/sbin/mini-buildd-debootstrap-uname-2.6\n"
-                        self.x_or_create(cb_class, source=source, architecture=architecture, defaults=defaults)
-                    except BaseException as e:
-                        LOG.warning(f"{source.codename}/{architecture.name}: Skipping chroot: {e}")
-
-            cb_class.Admin.mbd_pca_all(self.request)
-
-    def setup_remotes(self):
-        for ep in self.args["remotes"].value():
-            self.x_or_create(mini_buildd.mdls().gnupg.Remote, http=ep)
-        mini_buildd.mdls().gnupg.Remote.Admin.mbd_pca_all(self.request)
-
-    def setup_repositories(self):
-        # Create default layouts and suites.
+        # Layouts
         for layout_name, layout_setup in SETUP["layout"].items():
-            layout, layout_created = self.x_or_create(
-                mini_buildd.mdls().distribution.Layout,
-                name=layout_name,
-                defaults=layout_setup.get("options", {}))
-            if layout_created or self.args["update"].value():
-                for suite_name, suite_setup in layout_setup["suites"].items():
-                    suite, _suite_created = self.x_or_create(mini_buildd.mdls().distribution.Suite, name=suite_name)
+            instance = Setup.Instance(self, mini_buildd.mdls().distribution.Layout, layout_setup.get("options", {}), {}, name=layout_name)
+            for suite_name, suite_setup in layout_setup["suites"].items():
+                suite_instance = Setup.Instance(self, mini_buildd.mdls().distribution.Suite, {}, {}, name=suite_name)
+                Setup.Instance(self,
+                               mini_buildd.mdls().distribution.SuiteOption,
+                               {
+                                   **suite_setup.get("options", {}),
+                                   "extra_options": f"Rollback: {layout_setup['rollback'].get(suite_name, 0)}\n",
+                                   "migrates_to": mini_buildd.mdls().distribution.SuiteOption.mbd_get_or_none(layout=instance.obj, suite=mini_buildd.mdls().distribution.Suite.mbd_get_or_none(name=suite_setup.get("migrates_to"))),
+                               },
+                               {},
+                               layout=instance.obj,
+                               suite=suite_instance.obj)
 
-                    defaults = suite_setup.get("options", {})
-                    defaults["extra_options"] = f"Rollback: {suite_setup.get('rollback', 0)}\n" if layout_setup.get("with_rollbacks", False) else ""
-                    if suite_setup.get("migrates_to"):
-                        defaults["migrates_to"] = mini_buildd.mdls().distribution.SuiteOption.objects.get(layout=layout, suite=mini_buildd.mdls().distribution.Suite.objects.get(name=suite_setup.get("migrates_to")))
-
-                    self.x_or_create(mini_buildd.mdls().distribution.SuiteOption, layout=layout, suite=suite, defaults=defaults)
-
-        # Add default distribution objects for all base sources found.
-        for s in mini_buildd.mdls().source.Source.Admin.mbd_filter_active_base_sources():
-            codename_setup = CODENAMES[s.codename]
-            defaults = {
-                "apt_allow_unauthenticated": codename_setup.get("apt_allow_unauthenticated", False),
-                "extra_options": mini_buildd.sbuild.CONFIG_BLOCKS.default() + mini_buildd.sbuild.SETUP_BLOCKS.default(),
-            }
-            defaults["extra_options"] += codename_setup.get("extra_options", "")
-
-            distribution, distribution_created = self.x_or_create(mini_buildd.mdls().distribution.Distribution, base_source=s, defaults=defaults)
-
-            if distribution_created or self.args["update"].value():
+        # Distributions
+        class DistributionInstance(Setup.Instance):
+            def update(self):
                 # Auto-add known default components or Origin
-                distribution.components.set(mini_buildd.mdls().source.Component.objects.filter(name__in=SETUP["origin"][s.origin]["default_components"]))
+                self.obj.components.set(mini_buildd.mdls().source.Component.objects.filter(name__in=SETUP["origin"][self.obj.base_source.origin]["default_components"]))
 
                 # Auto-add extra (prio) sources that start with our base_source's codename
-                distribution.extra_sources.set(mini_buildd.mdls().source.PrioritySource.objects.filter(source__codename__regex=fr"^{s.codename}[-/]"))
+                self.obj.extra_sources.set(mini_buildd.mdls().source.PrioritySource.objects.filter(source__codename__regex=fr"^{self.obj.base_source.codename}[-/]"))
 
                 # Auto-add architectures to build for
                 archall_set = False  # Use first non-optional arch to build arch "all"
-                for architecture in s.architectures.all():
-                    if architecture.name in (mini_buildd.dist.Archs.native() + self.args["repositories_extra_architectures"].value()):
-                        optional = architecture.name in codename_setup.get("arch_optional", [])
-                        architecture_option, _dummy = self.x_or_create(
+                for architecture in self.obj.base_source.architectures.all():
+                    if architecture.name in (mini_buildd.dist.Archs.native() + self.call.args["repositories_extra_architectures"].value()):
+                        optional = architecture.name in self.update_args["setup"].get("distribution", {}).get("arch_optional", [])
+                        instance = Setup.Instance(
+                            self.call,
                             mini_buildd.mdls().distribution.ArchitectureOption,
-                            architecture=architecture,
-                            distribution=distribution,
-                            defaults={
+                            {
                                 "build_architecture_all": not archall_set and not optional,
                                 "optional": optional,
-                            })
-                        if architecture_option.build_architecture_all:
+                            },
+                            {},
+                            architecture=architecture,
+                            distribution=self.obj)
+                        if instance.obj.build_architecture_all:
                             archall_set = True
 
-                # Save changes to Distribution model
-                distribution.save()
+            def diff(self):
+                diff = super().diff()
+                if self.obj is not None:
+                    self._add_m2m_diff(diff, "components",
+                                       SETUP["origin"][self.obj.base_source.origin]["default_components"],
+                                       [c.name for c in self.obj.components.all()])
+                    self._add_m2m_diff(diff, "extra_sources",
+                                       [s.source.codename for s in mini_buildd.mdls().source.PrioritySource.objects.filter(source__codename__regex=fr"^{self.obj.base_source.codename}[-/]")],
+                                       [s.source.codename for s in self.obj.extra_sources.all()])
+                return diff
+
+        for codename in sources_codenames:
+            DistributionInstance(self,
+                                 mini_buildd.mdls().distribution.Distribution,
+                                 CODENAMES[codename].get("distribution", {}).get("options", {}),
+                                 {"setup": CODENAMES[codename]},
+                                 base_source=mini_buildd.mdls().source.Source.mbd_get_or_none(codename=codename))
+
+        # Repositories
+        class RepositoryInstance(Setup.Instance):
+            def update(self):
+                for d in mini_buildd.mdls().distribution.Distribution.objects.filter(**self.update_args["setup"]["distribution_filter"]):
+                    self.obj.distributions.add(d)
+
+            def diff(self):
+                diff = super().diff()
+                if self.obj is not None:
+                    self._add_m2m_diff(diff, "distributions",
+                                       [d.base_source.codename for d in mini_buildd.mdls().distribution.Distribution.objects.filter(**self.update_args["setup"]["distribution_filter"])],
+                                       [d.base_source.codename for d in self.obj.distributions.all()])
+                return diff
 
-        # Add repositories
         for repo_preset in self.args["repositories"].value():
             repo, dummy, preset = repo_preset.partition("/")
             preset = preset if preset else "default"
             repository_setup = SETUP["repository"][preset]
-            defaults = repository_setup.get("options", {})
-            defaults["layout"] = mini_buildd.mdls().distribution.Layout.objects.get(name__exact=repository_setup["layout"])
+            options = repository_setup.get("options", {})
+            options["layout"] = mini_buildd.mdls().distribution.Layout.mbd_get_or_none(name__exact=repository_setup["layout"])
+            RepositoryInstance(self, mini_buildd.mdls().repository.Repository, options, {"setup": repository_setup}, identity=repo)
 
-            repository, repository_created = self.x_or_create(mini_buildd.mdls().repository.Repository, identity=repo, defaults=defaults)
+        # Chroots
+        chroots_codenames = set(self.args["chroots"].value())
+        if self.args["chroots_from_sources"].value():
+            chroots_codenames.update(sources_codenames)
+        self._check_codenames(chroots_codenames)
 
-            if repository_created:
-                for d in mini_buildd.mdls().distribution.Distribution.objects.filter(**repository_setup["distribution_filter"]):
-                    repository.distributions.add(d)
+        cb_class = getattr(mini_buildd.mdls().chroot, f"{self.args['chroots_backend'].value()}Chroot")
+        for codename in chroots_codenames:
+            source = mini_buildd.mdls().source.Source.mbd_get_or_none(codename=codename)
+            for arch in mini_buildd.dist.Archs.native() + self.args["chroots_extra_architectures"].value():
+                architecture = Setup.Instance(self, mini_buildd.mdls().source.Architecture, {}, {}, name=arch)
+                Setup.Instance(self, cb_class, CODENAMES[codename].get("chroot", {}).get("options", {}), {}, source=source, architecture=architecture.obj)
 
-        mini_buildd.mdls().repository.Repository.Admin.mbd_pca_all(self.request)
+        # Remotes
+        for ep in self.args["remotes"].value():
+            Setup.Instance(self, mini_buildd.mdls().gnupg.Remote, {}, {}, http=ep)
 
     def run(self):
-        with mini_buildd.daemon.Stopped(force_start=True):
-            self.setup_daemon()
-            self.setup_archives()
-            self.setup_sources()
-            self.setup_chroots()
-            self.setup_remotes()
-            if self.args["repositories"].value():
-                self.setup_repositories()
-
-            if self.args["save"].value():
-                mini_buildd.get_daemon().model.mbd_setup_save(self.command_line())
+        self.result = []
+        if self._LOCK.acquire(blocking=False):  # pylint: disable=consider-using-with
+            try:
+                with mini_buildd.daemon.Stopped(force_start=True):
+                    self._setup()
+                    if self.args["save"].value():
+                        mini_buildd.get_daemon().model.mbd_setup_save(self.command_line(exclude=["update", "pca", "save"]))
+            finally:
+                self._LOCK.release()
+        else:
+            raise mini_buildd.HTTPUnavailable("API call 'setup' already running")
 
     @classmethod
     def _preset(cls, **kwargs):
-        return Setup(update=True, chroots_from_sources=True, **kwargs)
+        return Setup(update="all", chroots_from_sources=True, **kwargs)
 
     @classmethod
     def preset_debian(cls, **kwargs):
