@@ -52,16 +52,40 @@ from mini_buildd.builtins import mbd_cssmap
 
 LOG = logging.getLogger(__name__)
 
+MBD_MSGLEVEL2LOG = {
+    django.contrib.messages.DEBUG: logging.DEBUG,
+    django.contrib.messages.INFO: logging.INFO,
+    django.contrib.messages.SUCCESS: logging.INFO,
+    django.contrib.messages.WARNING: logging.WARNING,
+    django.contrib.messages.ERROR: logging.ERROR,
+}
 
-def mbd_message(request, exception, prefix="", level=django.contrib.messages.ERROR, level_followups=django.contrib.messages.WARNING):
-    """Convey exception to user via django messaging, with extra multiline ('followups') formatting support"""
+
+def mbd_msg(request, msg_lines, prefix="", level=django.contrib.messages.INFO, level_followups=django.contrib.messages.INFO):
+    """Convey message to user via django messaging, with extra multiline ('followups') formatting support"""
     if request is not None:
-        for s in str(mini_buildd.e2http(exception)).split("\n"):
-            django.contrib.messages.add_message(request, level, django.utils.html.format_html(f"{prefix}{s}"))
+        for s in msg_lines.split("\n"):
+            msg = f"{prefix}{s}"
+            LOG.log(MBD_MSGLEVEL2LOG[level], msg)
+            django.contrib.messages.add_message(request, level, django.utils.html.format_html(msg))
             level = level_followups
             prefix = "↳ "
 
 
+def mbd_msg_exception(request, exception, prefix=""):
+    mbd_msg(request, str(mini_buildd.e2http(exception)), prefix=prefix, level=django.contrib.messages.ERROR, level_followups=django.contrib.messages.WARNING)
+    mini_buildd.log_exception(LOG, prefix, exception)
+
+
+class UnixTextField(django.db.models.TextField):
+    r"""
+    Normalizes ``\r\n`` newlines (usually send by browsers) to Unix newlines ``\n``
+    """
+
+    def get_prep_value(self, value):
+        return value.replace("\r\n", "\n")
+
+
 class Model(django.db.models.Model):
     """
     Abstract father model for all mini-buildd models.
@@ -71,8 +95,8 @@ class Model(django.db.models.Model):
     """
 
     MBD_HELP_EXTRA_OPTIONS = ""  # Set this to non-empty in derived models to "activate"
-    extra_options = django.db.models.TextField(blank=True, editable=True,
-                                               help_text="Extra options (in the form ``<KEY>: <VALUE>``, one per line):")
+    extra_options = UnixTextField(blank=True, editable=True,
+                                  help_text="Extra options (in the form ``<KEY>: <VALUE>``, one per line):")
 
     # - 1.0.x: Daemon: Kept last packages as pickled data (base64 encoded).
     # - 2.0.x: Daemon: Used to save a command line for api call "setup".
@@ -80,6 +104,18 @@ class Model(django.db.models.Model):
     pickled_data = django.db.models.TextField(blank=True,
                                               verbose_name="Multi Purpose Field")
 
+    @classmethod
+    def mbd_get_or_none(cls, **kwargs):
+        """Get unambiguous instance or ``None`` if no such instance (``objects.get()`` w/o exception)"""
+        try:
+            return cls.objects.get(**kwargs)
+        except cls.DoesNotExist:
+            return None
+
+    @classmethod
+    def mbd_get_fields(cls, exclude=None):
+        return [f for f in cls._meta.get_fields() if f.name not in ([] if exclude is None else exclude)]
+
     class Meta():
         abstract = True
         app_label = "mini_buildd"
@@ -175,26 +211,6 @@ class Model(django.db.models.Model):
         self.full_clean()
         super().save(*args, **kwargs)
 
-    @classmethod
-    def mbd_diff(cls, obj, **defaults):
-        default = cls(**defaults)
-
-        diff = {}
-        for field_name in defaults:
-            try:
-                current_value = getattr(obj, field_name, None)
-                default_value = getattr(default, field_name, None)
-                if current_value != default_value:
-                    diff[field_name] = {
-                        "current": str(current_value),
-                        "default": str(default_value),
-                    }
-            except BaseException as e:
-                diff[field_name] = {
-                    "not_comparable": f"{field_name}: {e}",
-                }
-        return diff
-
     @classmethod
     def mbd_get_default(cls, field_name):
         return cls._meta.get_field(field_name).get_default()  # pylint: disable=protected-access
@@ -353,20 +369,17 @@ class StatusModel(Model):
             for o in queryset:
                 try:
                     getattr(cls, "mbd_" + action)(o, **kwargs)
-                    msg = f"{action} succeeded: {o}"
-                    LOG.info(msg)
-                    if request is not None:
-                        django.contrib.messages.add_message(request, django.contrib.messages.INFO, msg)
+                    mbd_msg(request, f"{action} succeeded: {o}")
                 except BaseException as e:
-                    msg = f"{action} failed: {o}"
-                    mini_buildd.log_exception(LOG, msg, e)
-                    mbd_message(request, e, msg + ": ")
+                    mbd_msg_exception(request, e, f"{action} failed: {o}")
 
         @classmethod
         def mbd_actions(cls, request, queryset, actions):
             for action in actions:
                 cls.mbd_action(request, queryset, action)
 
+        # Admin actions
+        # .. attention:: **compat** (django < 3.2): Drop monkey-patching ``short_description`` in favor of ``@django.contrib.admin.action(description="Foo")`` (https://docs.djangoproject.com/en/4.1/releases/3.2/#new-decorators-for-the-admin-site)
         def mbd_action_prepare(self, request, queryset):
             self.mbd_action(request, queryset, "prepare")
         mbd_action_prepare.short_description = "Prepare"
@@ -375,10 +388,18 @@ class StatusModel(Model):
             self.mbd_action(request, queryset, "check", force=True)
         mbd_action_check.short_description = "Check"
 
+        def mbd_action_pc(self, request, queryset):
+            self.mbd_actions(request, queryset, ["prepare", "check"])
+        mbd_action_pc.short_description = "PC"
+
         def mbd_action_activate(self, request, queryset):
             self.mbd_action(request, queryset, "activate")
         mbd_action_activate.short_description = "Activate"
 
+        def mbd_action_pca(self, request, queryset):
+            self.mbd_actions(request, queryset, ["prepare", "check", "activate"])
+        mbd_action_pca.short_description = "PCA"
+
         def mbd_action_deactivate(self, request, queryset):
             self.mbd_action(request, queryset, "deactivate")
         mbd_action_deactivate.short_description = "Deactivate"
@@ -400,18 +421,10 @@ class StatusModel(Model):
                     "action_checkbox_name": django.contrib.admin.helpers.ACTION_CHECKBOX_NAME})
         mbd_action_remove.short_description = "Remove"
 
-        def mbd_action_pc(self, request, queryset):
-            self.mbd_actions(request, queryset, ["prepare", "check"])
-        mbd_action_pc.short_description = "PC"
-
-        def mbd_action_pca(self, request, queryset):
-            self.mbd_actions(request, queryset, ["prepare", "check", "activate"])
-        mbd_action_pca.short_description = "PCA"
-
         @classmethod
-        def mbd_pca_all(cls, request):
-            """Run prepare, check, and activate for all objects of this model."""
-            cls.mbd_actions(request, cls.mbd_model.objects.all(), ["prepare", "check", "activate"])
+        def mbd_pca(cls, request, queryset):
+            """Run prepare, check, and activate for queryset of this model."""
+            cls.mbd_actions(request, queryset, ["prepare", "check", "activate"])
 
         @classmethod
         def colored_status(cls, obj):
