@@ -65,10 +65,10 @@ Will be used in:
 """)
 
     # GnuPG options
-    gnupg_template = django.db.models.TextField(default=("Key-Type: RSA\n"
-                                                         "Key-Length: 4096\n"
-                                                         "Expire-Date: 0\n"),
-                                                help_text="""\
+    gnupg_template = mini_buildd.models.base.UnixTextField(default=("Key-Type: RSA\n"
+                                                                    "Key-Length: 4096\n"
+                                                                    "Expire-Date: 0"),
+                                                           help_text="""\
 Template as accepted by 'gpg --batch --gen-key' (see 'man gpg').
 
 You should not give 'Name-Real' and 'Name-Email', as these are
@@ -167,8 +167,8 @@ maintainers to be accidentally spammed.
         mini_buildd.get_daemon().sync()
 
     def __str__(self):
-        return (f"{self.identity}: Serving {len(mini_buildd.models.repository.Repository.mbd_get_active())} repositories, "
-                f"{len(mini_buildd.models.chroot.Chroot.mbd_get_active())} chroots, "
+        return (f"Daemon '{self.identity}' serving {len(mini_buildd.models.repository.Repository.mbd_get_active())} repositories, "
+                f"entertaining {len(mini_buildd.models.chroot.Chroot.mbd_get_active())} chroots, "
                 f"using {len(mini_buildd.models.gnupg.Remote.mbd_get_active())} remotes")
 
     def mbd_setup_v10x2v20x(self):
