@@ -21,8 +21,8 @@ LOG = logging.getLogger(__name__)
 class GnuPGPublicKey(mini_buildd.models.base.StatusModel):
     key_id = django.db.models.CharField(max_length=100, blank=True, default="",
                                         help_text="Give a key id here to retrieve the actual key automatically per configured key server.")
-    key = django.db.models.TextField(blank=True, default="",
-                                     help_text="ASCII-armored GnuPG public key. Leave the key id blank if you fill this manually.")
+    key = mini_buildd.models.base.UnixTextField(blank=True, default="",
+                                                help_text="ASCII-armored GnuPG public key. Leave the key id blank if you fill this manually.")
 
     key_long_id = django.db.models.CharField(max_length=254, blank=True, default="")
     key_created = django.db.models.CharField(max_length=254, blank=True, default="")
@@ -39,7 +39,7 @@ class GnuPGPublicKey(mini_buildd.models.base.StatusModel):
         readonly_fields = ["key_long_id", "key_created", "key_expires", "key_name", "key_fingerprint"]
 
     def __str__(self):
-        return f"{self.key_long_id if self.key_long_id else self.key_id}: {self.key_name}"
+        return f"GnuPGPublicKey '{self.key_long_id if self.key_long_id else self.key_id}' ({self.key_name})"
 
     def clean(self, *args, **kwargs):
         super().clean(*args, **kwargs)
@@ -125,7 +125,7 @@ class Uploader(KeyringKey):
         filter_horizontal = ("may_upload_to",)
 
     def __str__(self):
-        return f"'{self.user}' may upload to '{','.join([r.identity for r in self.may_upload_to.all()])}' with key '{super().__str__()}'"
+        return f"Uploader '{self.user}' may upload to '{','.join([r.identity for r in self.may_upload_to.all()])}' with key '{super().__str__()}'"
 
 
 def cb_create_user_profile(sender, instance, created, **kwargs):
@@ -154,7 +154,7 @@ class Remote(KeyringKey):
         return mini_buildd.client.Client(self.mbd_url()).api(command, timeout=timeout)
 
     def __str__(self):
-        return self.mbd_url()
+        return f"Remote '{self.mbd_url()}' with {super().__str__()}"
 
     def mbd_get_status(self, timeout=2, wake=False, wake_sleep=5, wake_attempts=2):
         attempt = 0
