@@ -31,7 +31,7 @@ class EmailAddress(mini_buildd.models.base.Model):
         verbose_name_plural = "Email addresses"
 
     def __str__(self):
-        return f"{self.name} <{self.address}>"
+        return f"EmailAddress '{self.name} <{self.address}>'"
 
 
 class Repository(mini_buildd.models.base.StatusModel):
@@ -51,8 +51,8 @@ Debian backports.
     allow_unauthenticated_uploads = django.db.models.BooleanField(default=False,
                                                                   help_text="Allow unauthenticated user uploads.")
 
-    extra_uploader_keyrings = django.db.models.TextField(blank=True,
-                                                         help_text="""\
+    extra_uploader_keyrings = mini_buildd.models.base.UnixTextField(blank=True,
+                                                                    help_text="""\
 Extra keyrings, line by line, to be allowed as uploaders (in addition to configured django users).
 
 Example:
@@ -94,7 +94,7 @@ Example:
             return fields
 
     def __str__(self):
-        return f"{self.identity}: {' '.join([d.base_source.codename + ('' if d.mbd_is_active() else '*') for d in self.mbd_sorted_distributions()])}"
+        return f"Repository '{self.identity}' with distributions for {','.join([d.base_source.codename + ('' if d.mbd_is_active() else '*') for d in self.mbd_sorted_distributions()])}"
 
     def mbd_json(self):
         return {
@@ -210,7 +210,7 @@ Example:
         return mini_buildd.files.AptLine(mini_buildd.config.URIS["repositories"]["static"].url_join(),
                                          self.identity,
                                          diststr,
-                                         distribution.components.all(),
+                                         [c.name for c in distribution.components.all()],
                                          comment=f"{mini_buildd.get_daemon().model.mbd_get_archive_origin()} '{diststr}': {self.mbd_get_apt_pin(distribution, suite_option)}")
 
     def mbd_get_apt_build_sources_list(self, distribution, suite_option):
