@@ -70,7 +70,7 @@ class ConfigBlocks(Blocks):
         super().__init__("config")
 
     def default(self):
-        return f"{self.extra_option(top=False)}: ccache\n"
+        return f"{self.extra_option(top=False)}: ccache"
 
     def configure(self, file_, get, top):
         """Add configured blocks to provided file (space separated config option is auto-magically received via provided getter)."""
@@ -83,7 +83,7 @@ class SetupBlocks(Blocks):
         super().__init__("setup")
 
     def default(self):
-        return f"{self.extra_option(top=False)}: ccache eatmydata\n"
+        return f"{self.extra_option(top=False)}: ccache eatmydata"
 
     def configure(self, dir_, get, top):
         """Add configured blocks to provided dir (space separated config option is auto-magically received via provided getter)."""
