@@ -18,7 +18,7 @@ from django.views.decorators.http import require_GET
 from django.contrib.auth.decorators import login_required, user_passes_test
 
 import mini_buildd.api
-from mini_buildd.models.base import mbd_message
+from mini_buildd.models.base import mbd_msg_exception
 from mini_buildd.config import URIS
 
 LOG = logging.getLogger(__name__)
@@ -59,8 +59,7 @@ class ExceptionMiddleware():
     def error(cls, request, exception, output="html", status=http.HTTPStatus.INTERNAL_SERVER_ERROR):
         http_exception = mini_buildd.e2http(exception, status)
         if http_exception.status != http.HTTPStatus.OK:
-            mini_buildd.log_exception(LOG, "Middleware exception", exception)
-            mbd_message(request, exception)
+            mbd_msg_exception(request, exception, "Middleware exception")
 
         return django.template.response.TemplateResponse(request,
                                                          cls.MBD_TEMPLATES[output],
@@ -96,7 +95,7 @@ class TemplateView(django.views.generic.base.TemplateView):
         try:
             before_arg = self.request.GET.get("before", mini_buildd.misc.Datetime.now().isoformat())  # 'YYYY-MM-DDTHH:MM:SS' (isoformatted string). That's what we get from HTML 'datetime-local' input with no TZ meta (naive)
             before = mini_buildd.misc.Datetime.from_iso(before_arg)                                   # datetime object for internal use. Treat as UTC
-            return before, int(self.request.GET.get("limit", "100"))
+            return before, int(self.request.GET.get("limit", "40"))
         except Exception as e:
             raise mini_buildd.HTTPBadRequest("Invalid range parameters 'before', or 'limit'") from e
 
